package ca.canadapost.cpcdp.ncshipping;

import java.math.BigDecimal;
import java.util.Map;

import ca.canadapost.cpcdp.ncshipping.stub.ncshipment.*;

import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import ca.canadapost.cpcdp.ncshipping.util.ClientUtil;

import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;

/**
 * Sample code for the RequestNCShipmentRefund Canada Post service.
 * <p>
 * The RequestNCShipmentRefund service is used to request a refund for 
 * a shipment that has been transmitted. Making this call indicates 
 * that the previously printed label is spoiled or will otherwise not be used.
 * <p>
 * Note that these are requests for refund, as refunding a shipment requires 
 * proper verification before being actioned (i.e. ensure label has not been used, for example).
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 */

public class RequestNCShipmentRefundClient {
    // Main class for local testing only
    public static void main(String[] args) {    	
        try {
			// Your username, password and customer number are imported from the following file: CPCWS_SOAP_NCShipping_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");
            String mailedBy = userProperties.get("mailedBy");

            NCShipmentService service = new NCShipmentService();
            NCShipmentPortType port = service.getNCShipmentPort();

            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/rs/soap/ncshipment/v4");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));

            //build request
            NonContractShipmentRefundRequestType nonContractShipmentRefundRequestType = new NonContractShipmentRefundRequestType();
            nonContractShipmentRefundRequestType.setEmail("user@host.com");

            RequestNonContractShipmentRefundRequest request = new RequestNonContractShipmentRefundRequest();
            request.setLocale("FR");
            request.setMailedBy(mailedBy);
            request.setNonContractShipmentRefundRequest(nonContractShipmentRefundRequestType);

        	// Execute Request
        	RequestNonContractShipmentRefundResponse response = port.requestNCShipmentRefund(request);

            // Retrieve values from response object
            if (response.getMessages()  == null) {
            	System.out.println("Service Ticket ID: " + response.getNonContractShipmentRefundRequestInfo().getServiceTicketId());
            	System.out.println("Service Ticket Date: " + response.getNonContractShipmentRefundRequestInfo().getServiceTicketDate());
            } else {
            	// Assume Error
                for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }                                    
        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
